from pathlib import Path


def get_version(project_toml: Path) -> list[int]:
    content = project_toml.read_text(encoding="utf8")
    import re

    versions = re.findall(r'version = "([0-9\.]+)"', content)
    if len(versions) == 0:
        print("cannot find version")
        exit(-1)

    if versions[0].count(".") != 3:  # year, month, day, rev
        print("incorrect version format")
        exit(-1)

    year, month, day, rev = versions[0].split(".")
    try:
        year = int(year)
        month = int(month)
        day = int(day)
        rev = int(rev)
    except ValueError:
        print("incorrect version format")
        exit(-1)

    return [year, month, day, rev]


def next_version(version: list[int]) -> str:
    year, month, day, rev = version
    new_version = [year, month, day, rev]
    import datetime

    now = datetime.datetime.now()
    if year == now.year and month == now.month and day == now.day:
        new_version[3] += 1
    else:
        new_version = [now.year, now.month, now.day, 0]

    return new_version


def str_version(version: list[int]) -> str:
    if isinstance(version, str):
        return version
    return ".".join(map(str, version))


def update_version() -> str:
    from .toml import get_project_toml

    project_toml = get_project_toml()
    version = get_version(project_toml)
    nextv = next_version(version)

    old_version_str = str_version(version)
    new_version_str = str_version(nextv)

    old_line = f'version = "{old_version_str}"'
    new_line = f'version = "{new_version_str}"'

    content = project_toml.read_text(encoding="utf8")
    new_content = content.replace(old_line, new_line, 1)

    project_toml.write_text(encoding="utf8", data=new_content)

    return new_version_str



import subprocess
from .version import update_version  # noqa: F401

"steps to perform"


def build():
    subprocess.run(["uv", "build"], check=True)


def upload(version: str):
    subprocess.run(
        [
            "twine",
            "upload",
            "--repository",
            "pypi",
            f"./dist/*-{version}.*",
            f"./dist/*-{version}-*",
        ],
        check=True,
    )


def git_tag(version: str):
    subprocess.run(["git", "tag", version], check=True)


def git_commit(version: str):
    # git_stat = subprocess.getoutput("git status")
    # if not "working tree clean" in git_stat:
    #     print("please commit your changes first")
    subprocess.run(["git", "add", "."], check=True)
    subprocess.run(["git", "commit", "-m", f"bump version to {version}"])



from pathlib import Path

"""get project toml"""


def get_project_toml() -> Path:
    import os

    cwd = Path(os.getcwd())
    if not cwd.joinpath("pyproject.toml").exists():
        print("cannot find pyproject.toml")
        exit(-1)
    return cwd.joinpath("pyproject.toml")



import subprocess
from .args import get_args
from .steps import build, upload, git_tag, update_version, git_commit


def main():
    version = update_version()
    args = get_args()
    if args.build:
        build()
        if args.upload:
            upload(version)

    if args.git_commit:
        git_commit(version)

    if args.git_push:
        subprocess.run(["git", "push"])

    if args.git_tag:
        git_tag(version)

        if args.git_push:
            subprocess.run(["git", "push", "--tags"])


if __name__ == "__main__":
    main()



import argparse

"parse program arguments"


class Arg:
    build: bool = False
    'build package'

    upload: bool = False
    'upload package to twine (only in effect with build)'

    git_commit: bool = False
    'create a git commit'

    git_tag: bool = False
    'create a git tag'

    git_push: bool = False
    'push git changes (commit/tag)'


def get_args() -> Arg:
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--build", "-b", action="store_true", default=False, help="build package"
    )
    parser.add_argument(
        "--upload",
        "-u",
        action="store_true",
        default=False,
        help="upload package to pypi",
    )

    parser.add_argument(
        "--git-commit",
        "-c",
        action="store_true",
        default=False,
        help="create a git commit",
    )

    parser.add_argument(
        "--git-tag",
        "-t",
        action="store_true",
        default=False,
        help="create a git tag",
    )

    parser.add_argument(
        "--git-push",
        "-p",
        action="store_true",
        default=False,
        help="create a git tag",
    )

    args_obj = parser.parse_args()
    args = Arg()
    args.build = args_obj.build
    args.upload = args_obj.upload
    args.git_tag = args_obj.git_tag
    args.git_commit = args_obj.git_commit
    args.git_push = args_obj.git_push
    return args
